(*
    A simple metronome - MIDI utilities
    Copyright (c) 1998-2003 Alessandro Scotti
    www.ascotti.org

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*)
unit Midi;

interface

uses MMSystem;

const
  defaultMidiChannel = 9;

type
  TMidiTick = class
  private
    FMidiOut  : HMidiOut;
    FChannel  : byte;
    FDevice   : integer;
    FNote     : byte;
    procedure SetNote( Value: byte );
  public
    constructor Create( ADevice: integer; ANote, AChannel: byte );
    destructor Destroy; override;
    procedure Tick;
    property Channel: byte read FChannel write FChannel;
    property Device: integer read FDevice;
    property Note: byte read FNote write SetNote;
  end;

procedure PlayFrequency( Freq: integer );

implementation

uses Classes;

constructor TMidiTick.Create( ADevice: integer; ANote, AChannel: byte );
begin
  FChannel := AChannel;
  FDevice := ADevice;
  FNote := 0;
  if( midiOutOpen( @FMidiOut, ADevice, 0, 0, CALLBACK_NULL ) <> MMSYSERR_NOERROR ) then
    FMidiOut := 0;
  Note := ANote;
end;

destructor TMidiTick.Destroy;
begin
  if( FMidiOut <> 0 ) then
    midiOutClose( FMidiOut );
end;

procedure TMidiTick.SetNote( Value: byte );
var
  KeyArray: TKeyArray;
begin
  if( Value < 35 ) then Value := 35;
  if( Value > 81 ) then Value := 81;
  if( FNote <> Value ) then begin
    FNote := Value;
    if( FMidiOut = 0 ) then Exit;
    FillChar( KeyArray, sizeof(KeyArray), 0 );
    KeyArray[FNote] := 1 shl FChannel;
    midiOutCacheDrumPatches( FMidiOut, 0, @KeyArray, MIDI_CACHE_BESTFIT );
  end;
end;

procedure TMidiTick.Tick;
const
  NoteOn = $90;
  NoteOff = $80;
var
  D: longint;
begin
  if( FMidiOut = 0 ) then Exit;
  D := $007F0000 or (longint(FNote) shl 8) or FChannel;
  midiOutShortMsg( FMidiOut, D or NoteOn );
  midiOutShortMsg( FMidiOut, D or NoteOff );
end;

(*
  Experimental: build a WAV file in memory and play it.
*)
procedure PlayFrequency( Freq: integer );
const
  SampleRate = 44100;
const
  FCC_RIFF  = $46464952;        (* 'RIFF' *)
  FCC_LIST  = $5453494C;        (* 'LIST' *)
  FORM_WAVE = $45564157;        (* 'WAVE' *)
  FORM_WFMT = $20746D66;        (* 'fmt ' *)
  FORM_WDAT = $61746164;        (* 'data' *)
type
  TRiffChunk = packed record
    FCC   : longint;
    Size  : longint;
  end;
var
  O: TMemoryStream;
  H: TWaveFormat;
  S: smallint;
  R: TRiffChunk;
  W: longint;
  B: word;
  I: integer;
begin
  H.wFormatTag := WAVE_FORMAT_PCM; // PCM
  H.nChannels := 1;
  H.nSamplesPerSec := SampleRate;
  H.nAvgBytesPerSec := SampleRate*sizeof(S);
  H.nBlockAlign := sizeof(S);
  O := TMemoryStream.Create;
  R.FCC := FCC_RIFF;
  R.Size := sizeof(R) +         // 'RIFF' chunk
    sizeof(W) +                 // 'WAVE' identifier
    sizeof(R) +                 // 'fmt ' chunk
    sizeof(H) +                 // Wave format info
    sizeof(B) +                 // PCM-specific: bits/sample
    sizeof(R) +                 // 'data' chunk
    H.nAvgBytesPerSec;          // Data (1 sec)
  O.SetSize( R.Size );
  O.Write( R, sizeof(R) );
  W := FORM_WAVE;
  O.Write( W, sizeof(W) );
  R.FCC := FORM_WFMT;
  R.Size := sizeof(H) + sizeof(B);
  O.Write( R, sizeof(R) );
  O.Write( H, sizeof(H) );
  B := 16;
  O.Write( B, sizeof(B) );
  R.FCC := FORM_WDAT;
  R.Size := H.nAvgBytesPerSec;
  O.Write( R, sizeof(R) );
  for I:=0 to SampleRate-1 do begin
    S := Round( Sin( (I*2*Pi)/(SampleRate/Freq) )*32000 );
    O.Write( S, sizeof(S) );
  end;
  if( PlaySound( PChar(O.Memory), 0, SND_SYNC or SND_MEMORY ) ) then begin
  end;
  O.Free;
end;

end.

